#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
# coding: utf-8
from sqlalchemy import (Boolean, Column, DateTime, Float, ForeignKey, Index,
                        Integer, String, Text, text)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

Base = declarative_base()
metadata = Base.metadata


class Coauthor(Base):
    __tablename__ = "coauthors"
    __table_args__ = (Index("coauthors_idx_project_id_idx", "idx", "project_id"),)

    project_id = Column(ForeignKey("project.id"), primary_key=True, nullable=False, index=True)
    projectauthor_id = Column(ForeignKey("projectauthor.id"), nullable=False, unique=True)
    idx = Column(Integer, primary_key=True, nullable=False)

    project = relationship("Project")
    projectauthor = relationship("Projectauthor", uselist=False)


class Project(Base):
    __tablename__ = "project"

    id = Column(Integer, primary_key=True, server_default=text("nextval('project_id_seq'::regclass)"))
    rowversion = Column(Integer, nullable=False)
    projectcode = Column(String, nullable=False, unique=True)
    title = Column(String)
    proposalcode = Column(String, nullable=False)
    createdon = Column(DateTime)
    createdby = Column(Integer)
    lastupdatedon = Column(DateTime)
    lastupdatedby = Column(Integer)
    comments = Column(Text)
    projecttype = Column(String)
    test = Column(Boolean, server_default=text("false"))
    telescope = Column(String)
    refereemedian = Column(Float(53))
    has_submitted_sb = Column(Boolean, nullable=False, server_default=text("false"))
    is_complete = Column(Boolean, nullable=False, server_default=text("false"))
    hoursallocated = Column(Float(53), nullable=False)
    hoursusedsuccess = Column(Float(53), nullable=False)
    hoursusedfailure = Column(Float(53), nullable=False)
    pi = Column(ForeignKey("projectauthor.id"), index=True)
    contactauthor = Column(ForeignKey("projectauthor.id"), index=True)

    projectauthor = relationship("Projectauthor", primaryjoin="Project.contactauthor == Projectauthor.id")
    projectauthor1 = relationship("Projectauthor", primaryjoin="Project.pi == Projectauthor.id")


class Projectauthor(Base):
    __tablename__ = "projectauthor"

    id = Column(Integer, primary_key=True, server_default=text("nextval('projectauthor_id_seq'::regclass)"))
    rowversion = Column(Integer, nullable=False)
    globalid = Column(Integer, nullable=False)
    presentonproposal = Column(Boolean, nullable=False, server_default=text("false"))
    receivesemail = Column(Boolean, nullable=False, server_default=text("true"))
