#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
from workspaces.capability.services.capability_info import CapabilityInfo

pytest_plugins = ["testing.utils.conftest"]


def test_edit_capability(mock_capability_info: CapabilityInfo):
    """
    Tests that edit_capability method properly follows its logic
    """

    starting_flush_call_count = mock_capability_info.session.flush.call_count
    mock_capability_info.session.update.return_value = 1
    # Test edit with all inputs
    assert mock_capability_info.edit_capability("null", "test", 2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 1
    # Test edit with just name and steps
    assert mock_capability_info.edit_capability("null", steps="test")
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 2
    # Test edit with just name and max_jobs
    assert mock_capability_info.edit_capability("null", max_jobs=2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3
    # Test edit with just name
    assert mock_capability_info.edit_capability("null")
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3
    # Test "unable to edit" condition
    mock_capability_info.session.update.return_value = 0
    assert not mock_capability_info.edit_capability("null", "test", 2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3
