#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import abc
from abc import ABC
from typing import Optional

from workspaces.capability.enums import CapabilityStepType
from workspaces.products.schema_interfaces import ProductIF


class CapabilitySequenceIF(ABC):
    pass


class EventTranslatorIF(ABC):
    """
    Translates a Workflow event into a Capability event
    """

    pass


class CapabilityStepIF(ABC):
    """
    A step in a capability sequence
    """

    @property
    @abc.abstractmethod
    def step_type(self) -> CapabilityStepType:
        pass

    @property
    @abc.abstractmethod
    def step_value(self) -> Optional[str]:
        pass

    @property
    @abc.abstractmethod
    def step_args(self) -> Optional[str]:
        pass


class FutureCapabilityResultIF(ProductIF):
    """
    Class that represents a product that will in the future result from a completed
    capability execution
    """

    pass


class ParameterIF(ABC):
    """
    Class that represents a set of capability parameters, a required input for the correct
    and complete execution of a capability
    """

    def json(self) -> str:
        """
        Convert to json-formatted string
        :return: JSON string
        """
        raise NotImplementedError
